@@ -139,7 +139,8 @@ void libvisio::VSD6Parser::readCharIX(librevenge::RVNGInputStream *input)
   if (fontMod & 1) superscript = true;
   if (fontMod & 2) subscript = true;
 
-  input->seek(4, librevenge::RVNG_SEEK_CUR);
+  double scaleWidth = (double)(readU16(input)) / 10000.0;
+  input->seek(2, librevenge::RVNG_SEEK_CUR);
   double fontSize = readDouble(input);
 
   fontMod = readU8(input);
@@ -150,7 +151,7 @@ void libvisio::VSD6Parser::readCharIX(librevenge::RVNGInputStream *input)
   if (m_isInStyles)
     m_collector->collectCharIXStyle(m_header.id, m_header.level, charCount, font, fontColour, fontSize,
                                     bold, italic, underline, doubleunderline, strikeout, doublestrikeout,
-                                    allcaps, initcaps, smallcaps, superscript, subscript);
+                                    allcaps, initcaps, smallcaps, superscript, subscript, scaleWidth);
   else
   {
     if (m_isStencilStarted)
@@ -160,10 +161,10 @@ void libvisio::VSD6Parser::readCharIX(librevenge::RVNGInputStream *input)
 
     m_shape.m_charStyle.override(VSDOptionalCharStyle(charCount, font, fontColour, fontSize,
                                                       bold, italic, underline, doubleunderline, strikeout, doublestrikeout,
-                                                      allcaps, initcaps, smallcaps, superscript, subscript));
+                                                      allcaps, initcaps, smallcaps, superscript, subscript, scaleWidth));
     m_shape.m_charList.addCharIX(m_header.id, m_header.level, charCount, font, fontColour, fontSize,
                                  bold, italic, underline, doubleunderline, strikeout, doublestrikeout,
-                                 allcaps, initcaps, smallcaps, superscript, subscript);
+                                 allcaps, initcaps, smallcaps, superscript, subscript, scaleWidth);
   }
 }
 
@@ -303,7 +304,8 @@ void libvisio::VSD6Parser::readFillAndShadow(librevenge::RVNGInputStream *input)
       shadowOffsetY = m_shadowOffsetY;
     }
     m_shape.m_fillStyle.override(VSDOptionalFillStyle(colourFG, colourBG, fillPattern, fillFGTransparency,
-                                                      fillBGTransparency, shadowFG, shadowPattern, shadowOffsetX, shadowOffsetY));
+                                                      fillBGTransparency, shadowFG, shadowPattern,
+                                                      shadowOffsetX, shadowOffsetY, -1, -1, -1));
   }
 }
 
@@ -398,4 +400,48 @@ void libvisio::VSD6Parser::readTextField(librevenge::RVNGInputStream *input)
   }
 }
 
+void libvisio::VSD6Parser::readMisc(librevenge::RVNGInputStream *input)
+{
+  unsigned long initialPosition = input->tell();
+  unsigned char flags = readU8(input);
+  if (flags & 0x20)
+    m_shape.m_misc.m_hideText = true;
+  else
+    m_shape.m_misc.m_hideText = false;
+
+  input->seek(initialPosition+23, librevenge::RVNG_SEEK_SET);
+  while (!input->isEnd() && (unsigned long) input->tell() < (unsigned long)(initialPosition+m_header.dataLength+m_header.trailer))
+  {
+    unsigned long inputPos = input->tell();
+    unsigned length = readU32(input);
+    if (!length)
+      break;
+    unsigned blockType = readU8(input);
+    input->seek(1, librevenge::RVNG_SEEK_CUR);
+    if (blockType == 2)
+    {
+      if (0x74 == readU8(input))
+      {
+        if (0x6000004e == readU32(input))
+        {
+          unsigned shapeId = readU32(input);
+          if (0x7a == readU8(input))
+          {
+            if (0x40000073 == readU32(input))
+            {
+              if (!m_shape.m_xform1d)
+                m_shape.m_xform1d = new XForm1D();
+              if (m_shape.m_xform1d->beginId == MINUS_ONE)
+                m_shape.m_xform1d->beginId = shapeId;
+              else if (m_shape.m_xform1d->endId == MINUS_ONE)
+                m_shape.m_xform1d->endId = shapeId;
+            }
+          }
+        }
+      }
+    }
+    input->seek(inputPos + length, librevenge::RVNG_SEEK_SET);
+  }
+}
+
 /* vim:set shiftwidth=2 softtabstop=2 expandtab: */
