@@ -1,39 +1,18 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* libvisio
- * Version: MPL 1.1 / GPLv2+ / LGPLv2+
+/*
+ * This file is part of the libvisio project.
  *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License or as specified alternatively below. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * Major Contributor(s):
- * Copyright (C) 2011 Fridrich Strba <fridrich.strba@bluewin.ch>
- * Copyright (C) 2011 Eilidh McAdam <tibbylickle@gmail.com>
- *
- *
- * All Rights Reserved.
- *
- * For minor contributions see the git repository.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPLv2+"), or
- * the GNU Lesser General Public License Version 2 or later (the "LGPLv2+"),
- * in which case the provisions of the GPLv2+ or the LGPLv2+ are applicable
- * instead of those above.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-
+#include <string.h>
 #include "VSDInternalStream.h"
 
 
-VSDInternalStream::VSDInternalStream(WPXInputStream *input, unsigned long size, bool compressed) :
-  WPXInputStream(),
+VSDInternalStream::VSDInternalStream(librevenge::RVNGInputStream *input, unsigned long size, bool compressed) :
+  librevenge::RVNGInputStream(),
   m_offset(0),
   m_buffer()
 {
@@ -41,13 +20,12 @@ VSDInternalStream::VSDInternalStream(WPXInputStream *input, unsigned long size,
 
   const unsigned char *tmpBuffer = input->read(size, tmpNumBytesRead);
 
-  if (size != tmpNumBytesRead)
+  if (tmpNumBytesRead < 2)
     return;
 
   if (!compressed)
   {
-    for (unsigned long i=0; i<size; i++)
-      m_buffer.push_back(tmpBuffer[i]);
+    m_buffer.assign(tmpBuffer, tmpBuffer + tmpNumBytesRead);
   }
   else
   {
@@ -55,14 +33,14 @@ VSDInternalStream::VSDInternalStream(WPXInputStream *input, unsigned long size,
     unsigned pos = 0;
     unsigned offset = 0;
 
-    while (offset < size)
+    while (offset < tmpNumBytesRead)
     {
       unsigned flag = tmpBuffer[offset++];
-      if (offset > size-1)
+      if (offset > tmpNumBytesRead-1)
         break;
 
       unsigned mask = 1;
-      for (unsigned bit = 0; bit < 8 && offset < size; ++bit)
+      for (unsigned bit = 0; bit < 8 && offset < tmpNumBytesRead; ++bit)
       {
         if (flag & mask)
         {
@@ -72,7 +50,7 @@ VSDInternalStream::VSDInternalStream(WPXInputStream *input, unsigned long size,
         }
         else
         {
-          if (offset > size-2)
+          if (offset > tmpNumBytesRead-2)
             break;
           unsigned char addr1 = tmpBuffer[offset++];
           unsigned char addr2 = tmpBuffer[offset++];
@@ -102,11 +80,11 @@ const unsigned char *VSDInternalStream::read(unsigned long numBytes, unsigned lo
   numBytesRead = 0;
 
   if (numBytes == 0)
-    return 0;
+    return nullptr;
 
   int numBytesToRead;
 
-  if ((m_offset+numBytes) < m_buffer.size())
+  if (numBytes < m_buffer.size() - m_offset)
     numBytesToRead = numBytes;
   else
     numBytesToRead = m_buffer.size() - m_offset;
@@ -114,7 +92,7 @@ const unsigned char *VSDInternalStream::read(unsigned long numBytes, unsigned lo
   numBytesRead = numBytesToRead; // about as paranoid as we can be..
 
   if (numBytesToRead == 0)
-    return 0;
+    return nullptr;
 
   long oldOffset = m_offset;
   m_offset += numBytesToRead;
@@ -122,12 +100,14 @@ const unsigned char *VSDInternalStream::read(unsigned long numBytes, unsigned lo
   return &m_buffer[oldOffset];
 }
 
-int VSDInternalStream::seek(long offset, WPX_SEEK_TYPE seekType)
+int VSDInternalStream::seek(long offset, librevenge::RVNG_SEEK_TYPE seekType)
 {
-  if (seekType == WPX_SEEK_CUR)
+  if (seekType == librevenge::RVNG_SEEK_CUR)
     m_offset += offset;
-  else if (seekType == WPX_SEEK_SET)
+  else if (seekType == librevenge::RVNG_SEEK_SET)
     m_offset = offset;
+  else if (seekType == librevenge::RVNG_SEEK_END)
+    m_offset = long(static_cast<unsigned long>(m_buffer.size())) + offset;
 
   if (m_offset < 0)
   {
@@ -148,7 +128,7 @@ long VSDInternalStream::tell()
   return m_offset;
 }
 
-bool VSDInternalStream::atEOS()
+bool VSDInternalStream::isEnd()
 {
   if ((long)m_offset >= (long)m_buffer.size())
     return true;
